function [] = main_rti(u_max, Q, R, T, Tf, x_init, N)
clearvars -except u_max Q R T x_init Tf N 
% close all
clc

import casadi.*

% N = 50; % number of control intervals
nx = 2;
nu = 1;

% Declare model variables
x = MX.sym('x',nx,1);
u = MX.sym('u',nu,1);

% x_init = [0; 1];
x0_bar = MX.sym('x0_bar', 2, 1);

N_sim = Tf/(T/N); 

% Model equations
xdot = dynamics(x, u);

% Formulate discrete time dynamics
% Fixed step Runge-Kutta 4 integrator
M = 1; % RK4 steps per interval
DT = T/N/M;
f = Function('f', {x, u}, {xdot});
X0 = MX.sym('X0', nx,1);
U = MX.sym('U',nu,1);
X = X0;

for j=1:M
    k1 = f(X, U);
    k2 = f(X + DT/2 * k1, U);
    k3 = f(X + DT/2 * k2, U);
    k4 = f(X + DT * k3, U);
    X=X+DT/6*(k1 +2*k2 +2*k3 +k4);
end

F = Function('F', {X0, U}, {X});

% Evaluate at a test point
Fk = F([0.2; 0.3],0.4);
disp(Fk)

% Start with an empty NLP
w={};
w0 = [];
lbw = [];
ubw = [];
J = 0;
g={};
lbg = [];
ubg = [];

% Formulate the NLP
X0 = MX.sym('X0',2,1);
Xk = X0;
lbw = [lbw; -inf; -inf];
ubw = [ubw; inf; inf];

g = {g{:}, x0_bar-Xk};
w = {w{:}, Xk};
lbg = [lbg; 0; 0];
ubg = [ubg; 0; 0];
w0 = [w0;  x0_bar];

for k=0:N-1
    % New NLP variable for the control
    Uk = MX.sym(['U_' num2str(k)], nu, 1);
    w = {w{:}, Uk};
    w0 = [w0;  0.1];
    lbw = [lbw; -u_max];
    ubw = [ubw;  u_max];
    
    % Integrate till the end of the interval
    Xk_end =  F(Xk, Uk);
%     J = J+ norm(Xk)^2 + r*Uk^2;

    % New NLP variable for state at end of interval
    Xk = MX.sym(['X_' num2str(k+1)], nx, 1);
    w = {w{:}, Xk};
    lbw = [lbw; -inf; -inf];
    ubw = [ubw;  inf;  inf];
    w0 = [w0; 0.1; 0.1];

    % Add equality constraint
    g = {g{:}, Xk_end-Xk};
    lbg = [lbg; 0; 0];
    ubg = [ubg; 0; 0];
end

% J = J + 10*norm(Xk)^2;

w = vertcat(w{:});

g = vertcat(g{:});
G = Function('G', {w, x0_bar},{g});
JG = Function('JG', {w, x0_bar},{jacobian(g,w)});

% Define linearization point
wk = MX.sym('wk',length(w),1);
g_l = G(wk, x0_bar) + JG(wk, x0_bar)*(w - wk);

H = kron(eye(N), diag([diag(Q);diag(R)]));
H = diag([diag(H);diag(Q)]);
J = 1/2*w.'*H*w;

p_in = [wk;x0_bar];

% Allocate QP solver
qp = struct('x',w, 'f',J,'g',[g_l],'p',p_in);
solver = qpsol('solver', 'qpoases', qp);

iters = zeros(length(w),N_sim);
X_cl = zeros(2,N_sim);
U_cl = zeros(1,N_sim);
X_cl(:,1) = x_init;

% RTI solver
for i=1:N_sim    
    
    sol = solver('lbx',lbw, 'ubx', ubw, 'lbg', lbg, 'ubg', ubg , 'p', [iters(:,i);X_cl(:,i)]);
    sol = full(sol.x);
    iters(:,i+1) = sol;
    U_cl(i) = sol(3);
    X_cl(:,i+1) = full(F(X_cl(:,i), U_cl(i)));

end

% figure()
subplot(211)
hold all
plot([0:N_sim]*T/N, X_cl.','r--')
grid on
subplot(212)
hold all
stairs([0:N_sim-1]*T/N, U_cl,'r--')
grid on
end